package main;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.goodoldai.jeff.wizard.JEFFWizard;

import wine_selection_classes.WineRequest;

public class StartWineSelectionII {

	public static void main(String[] args) {
		
		//This parameter represents a focus point for all explanation features.
		JEFFWizard wizard;
		
		try {
			// load up the knowledge base
			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = kbase
					.newStatefulKnowledgeSession();
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory
					.newFileLogger(ksession, "test");
			
			// setting the parameters need for the expert system to reach its conclusion
			WineRequest wr = new WineRequest();
			wr.setConsumationTime("to accompany dessert");
			wr.setDessert("very sweet such as chocolate");
			
			// gets the wizard that will be used to create the explanation
			wizard = getJEFFWizardWithInternationalization();
			
			// greats the explanation and get ready for the building process
			wizard.createExplanation();

			// inserting parameters and starting the decision process 
			// which will also in the process build the explanation
			ksession.setGlobal("wizard", wizard);
			ksession.insert(wr);
			ksession.fireAllRules();

			if (wr.getRecommendedGenericWineType() != null)
				System.out.println("Recommended generic wine type: "
						+ wr.getRecommendedGenericWineType());
			if (wr.getSuggestedVarietalWine() != null)
				for (String wine : wr.getSuggestedVarietalWine()) {
					System.out.println("Varietal wine: " + wine);
				}
			logger.close();

			// creating the reports based on the given wizard
			createReports(wizard);

		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("wine_selection_rulesII.drl"),
				ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error : errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	/**
	 * Creates the JEFFWizard which will use the internationalization in the process
	 * 
	 * @return JEFFWizard represent a focus point for all explanation features.
	 */
	private static JEFFWizard getJEFFWizardWithInternationalization() {
		return new JEFFWizard("Bojan Tomic", null, null, "Wine recommendation", true);
	}
	
	/**
	 * Generates the reports form the provided wizard. 
	 * It will generate the xml, txt and pdf report.
	 * 
	 * @param wizard from which the report will be generated
	 */
	private static void createReports(JEFFWizard wizard) {
		
		// creates txt report 
		wizard.generateTXTReport("textReport.txt",false);
		// creates pdf report 
		wizard.generatePDFReport("pdfReport.pdf",false);
		// creates xml report 
		wizard.generateXMLReport("xmlReport.xml",false);
	}
}
